#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_UPGRADE_RESTRICTED="page_upgrade_restricted"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGES_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB"
PAGES_UPGRADE="$PAGE_UPGRADE_RESTRICTED $PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB"
PAGES_UNINST="$PAGE_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

IsUpgradable()
{
	true
}

NeedMigrateMDB5()
{
	local prev_version="$1"
	if [ -z "$prev_version" ] || [ "$prev_version" -le "$MIGRATE_DB_VERSION" ]; then
		true
		return
	fi
	false
	return
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

GetPreviousPkgBuildNumber()
{
	local old_pkg_ver="$SYNOPKG_OLD_PKGVER"
	if [ -z "${old_pkg_ver}" ]; then
		# SYNOPKG_OLD_PKGVER does not exist in un-install stage of upgrade
		old_pkg_ver="$SYNOPKG_PKGVER"
	fi

	local old_build_number=$(echo "$old_pkg_ver" | cut -d- -f2)
	echo "$old_build_number"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

wizard_db_settings="設定 DokuWiki 資料庫"
wizard_db_name_desc="請輸入 DokuWiki 資料庫名稱。"
wizard_db_name_label="資料庫名稱"
wizard_m10_desc="請輸入 MariaDB 10 管理者的帳號密碼，以進行 DokuWiki 資料庫設定。"
wizard_admin_acc="帳號"
wizard_admin_pass="密碼"
wizard_set_db_desc="請為 DokuWiki 建立資料庫專屬帳號，以使用原有/全新資料。"
wizard_set_data_title="設定 DokuWiki"
wizard_found_backup="套件保留資料不相容"
wizard_warn_data_backup_deprecated="此版本與您系統內的此套件保留資料不相容。<br>若您安裝此版本，保留資料將會被刪除。<br>如需留存保留資料，請安裝與前一次相同版本的套件。(先前版本：{prev_version})"
wizard_db_user_account_desc="資料庫使用者帳號"
wizard_db_user_password_desc="資料庫使用者密碼"
wizard_migrate_note="最新版本的 DokuWiki 僅支援 MariaDB 10。<br>系統已偵測到 MariaDB 5 的資料庫，在下個步驟中，系統會將 MariaDB 5 資料庫轉移至 MariaDB 10。<br><br>請輸入 MariaDB 5 管理者的帳號密碼，以進行安裝。"
wizard_migrate_title="轉移 DokuWiki 資料庫"
wizard_remove_msql_title="移除 DokuWiki 資料庫"
wizard_remove_msql_desc="若移除 DokuWiki 資料庫，全部資料將被刪除。"
wizard_msql_password_desc_remove="請輸入資料庫管理者的帳號密碼來移除 DokuWiki 資料庫。"
wizard_admin_acc="帳號"
wizard_admin_pass="密碼"
wizard_unable_to_upgrade_title="無法更新至此版本"
wizard_upgrade_from_prev_release_required_desc="若要更新套件至此版本，請先更新至<a href=\"{1}\" target=\"_blank\">前一版本</a>。"
wizard_decide_restore="DokuWiki 資料已存在。<br>請選擇下方任一動作："
wizard_create_new="全新安裝 (原有資料將全部被移除。)"
wizard="全新安裝 (原有資料將全部被移除。)"
wizard_remove_data_title="移除 DokuWiki 中的資料"
wizard_remove_data_desc="如果您移除 DokuWiki 中的資料，所有的文章和設定都會被刪除。"
wizard_remove_data_title="移除 DokuWiki 中的資料"
#!/bin/bash
# Copyright (c) 2000-2017 Synology Inc. All rights reserved.

PKG_NAME="DokuWiki"
PKG_DIR="dokuwiki"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"
PAGE_INST="$PAGE_RESTORE $MAIN_INSTALL"
PAGE_UPGRADE="$MAIN_UPGRADE"
CONF_FILE="conf/local.php"

CustomSetPages()
{
	NEW_INSTALL_PAGES=""
	RESTORE_PAGES="$(PageRestore)"
}

CustomHasRunWebsiteSetup()
{
	# initial setup for Dokuwiki is optional, it does not indicate the need for upgrade migration
	# there is no difference between setup-done and setup-required
	true
}

# workaround for not creating wizard
MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	
	# skip uninstall wizard
	if [ "$main_function" == "main_uninst" ]; then
		return
	fi
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" > "${output}_${lan}.sh"
		cat "lang/$lan" "wizard_customized" $pages "$main_function" >> "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

PageM5()
{
cat << EOF
{
	"step_title": "$wizard_migrate_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_migrate_note",
		"subitems": [{
			"key": "wizard_m5_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m5_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}

PageRestore()
{
	local prev_version="unknown"
	if [ -n "${RESERVED_DATA_PKG_VER}" ]; then
		prev_version="${RESERVED_DATA_PKG_VER}"
	fi
	local reserved_data_deprecate_msg=$(echo $wizard_warn_data_backup_deprecated | sed "s/{prev_version}/$prev_version/g")
cat << EOF
{
	"step_title": "$wizard_found_backup",
	"items": [{
		"desc": "$reserved_data_deprecate_msg"
	}]
}
EOF
}

PageM10()
{
cat << EOF
{
	"step_title": "$wizard_set_data_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_m10_desc",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" == item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		$(CustomPageActivate)

		if (arguments[0].headline == \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			db_user_textfield = null;
			grant_user_multiselect = null;
			mariadb_ver_textfield = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_user_account" "db_user_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")
			$(FindObj "mariadb_ver" "mariadb_ver_textfield")

			old_db = \"@OLD_DB@\";
			db_name = \"$INIT_DB_NAME\";
			db_user = \"$INIT_DB_USER\";

			old_db_textfield.setValue(old_db);
			new_db_textfield.setValue(db_name);
			new_db_textfield.setDisabled(false);
			db_user_textfield.setValue(db_user);
			mariadb_ver_textfield.setValue(\"$MARIADB_VER\");

			need_migrate_multiselect.setVisible(false);
			create_db_multiselect.setVisible(false);
			drop_db_multiselect.setVisible(false);
			grant_user_multiselect.setVisible(false);

		}

	}",
EOF
}

Deactivate()
{
cat << EOF
	"deactivate": "{
		if (arguments[0].headline == \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			create_db_collision_textfield = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			grant_user_multiselect = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "create_db_collision" "create_db_collision_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")

			table = {
				\"new5to10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": true, \"create_db_collision\": \"error\"},
				\"new10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": old_db_textfield.rawValue && (old_db_textfield.rawValue !== new_db_textfield.rawValue)? true : false, \"create_db_collision\": old_db_textfield.rawValue === new_db_textfield.rawValue? \"replace\" : \"error\"}
			};

			if ($NEED_MIGRATE_MDB5) {
				state = \"new5to10\";
			}
			else {
				state = \"new10\";
			}

			need_migrate_multiselect.setValue(table[state][\"migrate\"]);
			create_db_multiselect.setValue(table[state][\"create\"]);
			grant_user_multiselect.setValue(table[state][\"grant_user\"]);
			drop_db_multiselect.setValue(table[state][\"drop_db\"]);
			create_db_collision_textfield.setValue(table[state][\"create_db_collision\"]);
		}
}",
EOF
}

ApplyDBInfo()
{
	local page_db="$1"

	sed "s/@OLD_DB@/$DB_NAME/g;s/@OLD_USER@/$DB_USER/g" <<< "$page_db"
}

PageDB()
{
	local page_db=$(cat << EOF
{
	"step_title": "$wizard_db_settings",
	$(Activate)
	$(Deactivate)
	"items": [{
		"type": "textfield",
		"desc": "$wizard_set_db_desc",
		"subitems": [{
			"key": "pkgwizard_db_name",
			"desc": "$wizard_db_name_label",
			"disabled": true,
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_account",
			"desc": "$wizard_db_user_account_desc",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_password",
			"desc": "$wizard_db_user_password_desc"
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "create_db_collision",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "need_migrate",
			"desc": "migrate or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "create_db_flag",
			"desc": "create db or not",
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "old_db_name",
			"desc": "old db name",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "drop_db_flag",
			"desc": "drop old db or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "grant_user_flag",
			"desc": "must grant user if exist wizard",
			"defaultValue": true,
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "mariadb_ver",
			"desc": "mariadb version",
			"defaultValue": "$MARIADB_VER",
			"hidden": true
		}]
	}]
}
EOF
)
	ApplyDBInfo "$page_db"
}
SetupInstallVariables()
{
	NEED_MIGRATE_MDB5=false
	MARIADB_VER="m10"

	CustomSetBackupPath # set BACKUP_PATH
	CustomMainInstall

	DB_NAME=$(CustomGetDBName "$BACKUP_WEB_PATH")
	DB_USER=$(CustomGetDBUser "$BACKUP_WEB_PATH")
	RESERVED_DATA_PKG_VER="$(CustomGetValueFromMetaFile "version" | cut -s -d- -f2)"

	WebsiteSetupDone=false
	if CustomHasRunWebsiteSetup "$BACKUP_PATH"; then
		WebsiteSetupDone=true
		if NeedMigrateMDB5 "$RESERVED_DATA_PKG_VER"; then # check if needing to migrate MariaDB 5
			NEED_MIGRATE_MDB5=true
			MARIADB_VER="m5"
		fi
	fi
}

SetupInstallWizardPages()
{
	CustomSetPages # set: MIGRATE_PAGES RESTORE_PAGES NEW_INSTALL_PAGES

	local install_page=""
	if $WebsiteSetupDone; then
		if $NEED_MIGRATE_MDB5; then
			install_page="$MIGRATE_PAGES"
		else
			install_page="$RESTORE_PAGES"
		fi
	else
		install_page="$NEW_INSTALL_PAGES"
	fi

	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"
}

main()
{
	SetupInstallVariables
	SetupInstallWizardPages
	return 0
}

main "$@"

